# $Id: config.py,v 1.5 2001/03/13 13:14:15 olafb Exp $

#  Copyright 2001 Trustix As - <http://www.trustix.comQ>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


"""Module contains main function 'parse(filename=None)' for parsing
config for swup. Function returns a dictionary of config variables. If
a valid optional filename is given, variables defined in file will
override the corresponding default variables.

If parsing is not successful, a 'config.parse_error' is raised."""

import re, string, os


# exceptions
parse_error = 'config.parse_error'


# defaults
# you should use a config file to alter these values.
SWUP_DEFAULTS = {
    "pkgdriver": "pkgdriver.rpmdriver",
    "config_file" :  "/etc/swup/swup.conf",
    "logfile" : "/var/log/swup",
    "gnupgdir" : "/etc/swup/gnupg",
    "dependency_maxdepth" : "5",
    "cachedir" : "/var/cache/swup",
    "interactive_logged": "yes",
    "download_only" : "no",
    "save_to": "",
    "poll_only" : "no",
    "listsdir": "/var/cache/txpi/lists",
    "md5sumfile": "/var/cache/txpi/lists/md5sums",
    "dry_run" : "no",
    "quiet": 0,
    "verbose": 0,
    "tmpdir" : "/var/spool/swup",
    "flush_cache": "yes",
    "sites" :  [("0",
       "ftp://ftp.trustix.net/pub/Trustix/trustix-1.2/i586/Trustix/rdfs/latest.rdf",
                 ".*")],
    "exclude_pkg_regexp" : "kernel.*|glibc.*|ldconfig.*|rpm",
    "exclude_group_regexp" : None,
    "include_pkg_regexp": None,
    "include_group_regexp" : ".*"
    }


# regexps
removeline_regexp = re.compile(r"^(\s*$|\s*\#+)")
list_regexp = re.compile("^ *\[")
None_regexp = re.compile("^ *None")
string_regexp = re.compile(r"^ *(\"|')")
# Creating a regexp used to match for variables.
default_keys = SWUP_DEFAULTS.keys()
i = 0
var_regexp = "^(%s *=" % default_keys[i]
while i < (len(default_keys) -1):
    i = i + 1
    var_regexp = "%s|%s *=" % (var_regexp, default_keys[i])
var_regexp = "%s)" % var_regexp
var_regexp = re.compile(var_regexp)


def parse(filename=None):
    """Parse config and return a dictionary of config variables. If optional,
    'filename' is given, and a regular file 'filename' can be found in path,
    parse file and override variables specified in file."""
    dict = SWUP_DEFAULTS
    if filename:
        if os.path.isfile(filename):
            file = open(filename, "r")
            lines = file.readlines()
            file.close()
            dict.update(_parse_lines(lines))
        else:
            raise parse_error, "no such file %s" % filename
    return dict
        


def _clean_strings(strings):
    nstrings = len(strings)
    if nstrings < 1:
        return None

    # Check if this is a variable value of type 'list'.
    if list_regexp.match(strings[0]):
        new_s = ""
        for s in strings:
            new_s = new_s + s
        try:
            exec "list =" + new_s
        except:
            raise parse_error, str(new_s)
        return list
        
    # Else if the variable is a normal string.
    elif string_regexp.match(strings[0]):
        new_s = ""
        for s in strings:
            new_s = new_s + s
        try:
            exec "s =" + new_s
        except:
            raise parse_error, str(new_s)
        return s

    # Else if the variable is None.
    elif None_regexp.match(strings[0]):
        return None

    # Anything else will raise an exception.
    else:
        raise parse_error, "bad syntax, bailing out."

    
def _parse_lines(lines):

    vars = {}
    var = None
    stack = []
    nlines = len(lines)
    
    for i in range(nlines):
        line = lines[i]
        if (not i ==(nlines-1)) and removeline_regexp.match(line):
            # removing line
            continue
        elif var_regexp.match(line):
            if var:
                vars[var] = _clean_strings(stack)
                stack = []
            (var, value) = string.split(line, "=", 1)
            var = string.strip(var)
            line = value        
        stack.append(line)
        if i == (nlines -1):
            # This is the last line.
            if var:
                vars[var] = _clean_strings(stack)
    return vars



