#!/usr/bin/env python
# $Id: download.py,v 1.16 2001/03/13 13:51:20 olafb Exp $

#  Copyright 2001 Trustix As - <http://www.trustix.comQ>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


import sys, time, urllib, os, urlparse, string
import gpg


# some constants/defaults
MD5SUMFILE = '/var/cache/txpi/lists/md5sums' 
CACHEDIR = '/var/cache/swup'
TMPDIR = '/var/spool/swup/tmp'
SIGNATURE_APPEND = '.asc'
false, true = 0, 1




def byte_convert( bytes ):
    """Returns a two tuple (<str>, <prefix>), where prefix is one of
    '', 'k' or 'M'."""
    div = bytes/float(1024)
    if div < 1:
        return (str(bytes), '')
    if div < 1024:
        return ( "%4.2f" %div, 'k')
    else:
        div = bytes / float(1024**2)
        return ("%4.2f" %div, 'M')




class download:

    # define some exceptions
    download_error = 'download.download_error'
    signature_download_error = 'download.signature_download_error'
    signature_invalid_error = 'download.signature_invalid'
    flush_cache_error = 'download.flush_cache_error'
    auth_error = 'download.auth_error'
    
    # the general all-powerful exception
    error = (download_error, signature_download_error,
	     signature_invalid_error, flush_cache_error, auth_error)


    def __init__( self, log, cachedir=CACHEDIR, tmpdir=TMPDIR,
                  md5sumfile=MD5SUMFILE ):
        self.log = log
	self.cachedir = cachedir
	self.tmpdir = tmpdir
        self.md5sumfile = md5sumfile
        
	if not os.path.isdir( tmpdir ):
	    if os.path.exists( tmpdir ):
		os.remove( tmpdir )
            os.makedirs( tmpdir )

            
    def flush_cache( self ):
	urllib.urlcleanup()
	try:
	    if os.path.isdir(self.cachedir):
		from shutil import rmtree
		rmtree(self.cachedir)
		os.mkdir(self.cachedir, 0755)
	except:
	    raise self.flush_cache_error, "Unable to flush cache."


    def callback(self, blocknum, blocksize, filesize):
        #if size == -1: don't use size in calculation.

        if blocknum == 0:
            self._bytes_read = 0
            self._time_used = 0
            self._time = time.time()

        self._bytes_read = self._bytes_read + blocksize

        if filesize not in [-1,0]:
            if self._bytes_read > filesize:
                percent = 100
                self._bytes_read = filesize
            else:
                tmptime = time.time()
                self._time_used = self._time_used + (tmptime - self._time)
                self._time = tmptime
                percent = self._bytes_read / float(filesize) * 100
            if not self._time_used <= 0:
                (speed, speed_prefix) = byte_convert( self._bytes_read/
                                                      float(self._time_used))
            else:
                speed, speed_prefix = 0, 'k'
            
            (size, size_prefix) = byte_convert( filesize )
            (read, read_prefix) = byte_convert( self._bytes_read )
            message = "\rDownloading "\
                      "%-24s (%-7s%-1sb of %-7s%-1sb, %-3d%%, %-7s%-1sb/s)"%\
                      (self._now_downloading[:21]+"...", read, read_prefix,
                       size, size_prefix, percent, speed, speed_prefix)
        else:
            (read, read_prefix) = byte_convert( self._bytes_read )
            message = "\rDownloading %-56s (%-7s%-1sb)" %\
                      (self._now_downloading[:27], read, read_prefix)
        
        self.log.write_tty(message)



    def _do_download( self, uri ):
	parsed_uri = urlparse.urlparse( uri )
	(scheme, netloc, path, param, query, fragment) = parsed_uri
	if (not scheme == "file") and ("@" in netloc):
            raise self.auth_error, 'Authentication not supported.'
	else:
	    tmpuri = uri

        # Make sure the cache is ready to accept the file.
	filename = os.path.normpath(os.path.join(self.cachedir, uri))
	cachepath = os.path.dirname(filename)
	if not os.path.isdir(cachepath):
	    if os.path.exists(cachepath):
		os.remove(cachepath)
            os.makedirs(cachepath)

        basename = os.path.basename( uri )
        self._now_downloading = basename
	tmpfilename = os.path.join( self.tmpdir, basename )
        try:
            (retrfile, headers) = urllib.urlretrieve( tmpuri, tmpfilename,
                                                       self.callback )
        except KeyboardInterrupt:
            raise KeyboardInterrupt
        except:
            raise self.download_error, 'Unable to download file %s' \
                  % uri
        
        # gunzip compressed files
        if retrfile[-3:] == '.gz':
            import gzip
            gfd = gzip.open(retrfile)
            localfilename = retrfile[:-3]
            fd = open(localfilename, 'wb')
            block = gfd.read(8192)
            while block:
                fd.write(block)
                block = gfd.read(8192)
            gfd.close()
            fd.close()
            del(gfd, fd)
            os.remove(retrfile)
        else:
            localfilename = retrfile
            
	return localfilename

    def md5sumcheck(self, url, filename):
        'Check md5sum for filename in url.'
        old_md5s = {}
        new_md5s = {}
        if os.path.isfile(self.md5sumfile):
            fd = open(self.md5sumfile)
            line = fd.readline()
            while line:
                md5, file = string.split(line)
                line = fd.readline()
                old_md5s[file] = md5
            fd.close()
        try:
            tmpfile = self._do_download(url)
        except error:
            return 0
        
        fd = open(tmpfile)
        line = fd.readline()
        try:
            while line:
                md5, file = string.split(line)
                file = os.path.join(os.path.dirname(url), file)
                line = fd.readline()
                new_md5s[file] = md5
            fd.close()
        except ValueError:
            return 0
        
        isequal = 0
        key = filename
        if old_md5s.has_key(key):
            if new_md5s[key] == old_md5s[key]:
                
                isequal = 1

        # If the md5sum has changed, store new md5sums.
        if not isequal:
            old_md5s.update(new_md5s)
            md5sumdir = os.path.dirname(self.md5sumfile)
            if not os.path.isdir(md5sumdir):
                os.makedirs(md5sumdir)
            fd = open(self.md5sumfile, 'w')
            for key in old_md5s.keys():
                md5 = old_md5s[key]
                fd.write('%s  %s\n' % (md5, key))
            fd.close()
        return isequal

    def download( self, url, sigurl = None ):
	"""Download a file and signature, check signature, move to cachedir.

	The argument is the url of the file. Optional argument is the url
	of the signature. The default value for the signature name is
	url.asc"""

	# Check if the file exists - if it does, return
	if urlparse.urlparse(url)[0] == "":
	    url = "file:" + url
	filename = os.path.normpath(os.path.join(self.cachedir, url))
	cachepath = os.path.dirname(filename)

	if os.path.exists(filename):
	    # if we already have the file, don't download again...
	    return filename


	# Actually download stuff
	if sigurl == None:
            if url[-3:] == '.gz':
                sigurl = url[:-3] + SIGNATURE_APPEND
            else:
                sigurl = url + SIGNATURE_APPEND

        try:
            signame = self._do_download( sigurl )
        except:
            raise self.signature_download_error,\
                  "Unable to download signature for %s" % url
        try:
            tmpfilename = self._do_download( url )
        except:
            raise self.download_error, "Unable to download %s" % url
        
	# Check the sig
	try:
	    retval, output = gpg.checksig(signame)
	except gpg.signature_fail:
	    raise self.signature_invalid, sys.exc_info()[1]

	# If we have come this far, we should have the correct file properly
	# downloaded. Time to move it...
        os.rename(tmpfilename, filename)
	os.remove(signame)

	# return just the filename - we don't really care about the output...
	return filename
