#!/usr/bin/env python
# $Id: gpg.py,v 1.2 2001/01/11 09:50:03 olafb Exp $
# simple gpg wrapper by oysteivi

#  Copyright 2001 Trustix As - <http://www.trustix.comQ>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os

# define some defaults
gnupgdir = '/etc/swup/gnupg'
true, false = 1, 0

# define some exceptions
signature_fail = 'gpg.signature_fail'
key_import_failure = 'gpg.key_import_failure'
list_keys_failure = 'gpg.list_keys_failure'

# A general error which includes all availible errors
# nice as a security net or for lazy programmers...
general_error = \
(signature_fail, key_import_failure, list_keys_failure)


def checksig (filename, gnupgdir = gnupgdir):
	"""Checks the signature for a specified file.
	
	Takes the filename of the signature and an optional gnupg homedir.
	Returns true and the output of the gpg command (tuple) if the
	signature was verifiable, raises hell if not.
	"""
	
	command = "gpg --no-tty --homedir " + \
	gnupgdir + " --verify " + "'%s'" %filename  + " 2>&1"
	
	fd = os.popen(command)
	output = fd.read()
	retval = fd.close()
	
	if retval == None:
		return true, output
	else:
		raise signature_fail, output



def import_key (filename, gnupgdir = gnupgdir):
	"""Adds a key to the gnupg pubring.

	The filename of the key is the first argument. The second is 
	optionally the gnupg homedir to be used.

	If the key was already imported, the function is successful, but 
	nothing happens.
	
	Returns true, and the output of the gpg command, if the key was 
	successfully added, raises an exception if not.
	"""
	
	command = "gpg --no-tty --homedir " + \
		  gnupgdir + " --import " + filename + " 2>&1"

	fd = os.popen(command)
	output = fd.read()
	retval = fd.close()
	if retval == None:
		return true, output
	else:
		raise key_import_failure, output



def list_keys (gnupgdir = gnupgdir):
	"""List keys in gnupg pubring. Takes one optional argument to
	specify gnupg homedir where keys are found.
	"""

	command = "gpg --no-tty --homedir " + \
		  gnupgdir + " --list-public-keys " + "2>&1"
	fd = os.popen(command)
	output = fd.read()
	retval = fd.close()

	if retval == None:
		return true, output
	else:
		raise list_keys_failure, output
