#!/usr/bin/env python

#  Copyright 2001 Trustix As - <http://www.trustix.comQ>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, string, os
import help, cmdline, config, log

true, false = 1, 0

def main(args):
	
    # parse the command line
    try:
        argdict = cmdline.parse(args)
    except cmdline.unknown, option:
        print "swup: Unknown option:", option
	sys.exit(2)
    except cmdline.wrong_number_arguments, switch:
        print "swup: Wrong number of arguments for switch:", switch
	sys.exit(2)
    except cmdline.too_many_operations, (operation1, operation2):
        print "swup: Too many operations:", operation1, operation2
	sys.exit(2)
    except cmdline.too_few_operations, message:
        print "swup: Too few operations:", message
	print "swup: Use 'swup --help' for more information."
	sys.exit(2)


    # did user request help or version info?
    if argdict.has_key('--help'):
        help.helpme()
	sys.exit(0)
    if argdict.has_key('--version'):
	help.version()
	sys.exit(0)

    # get config
    if argdict.has_key('--config-file'):
	config_file = argdict['--config-file'][0]
    else:
	config_file = config.SWUP_DEFAULTS['config_file']

    configuration = None
    try:
	configuration = config.parse(config_file)
    except config.parse_error, message:
	print "swup: Error in config file", config_file
	print "swup: parser said: ", message

    # merge command line options with config
    if argdict.has_key('--silent'):
	configuration['loglevel'] = log.LOGLEVEL_SILENT 
    elif argdict.has_key('--quiet'):
        configuration['loglevel'] = log.LOGLEVEL_QUIET
    elif argdict.has_key('--verbose'):
        configuration['loglevel'] = log.LOGLEVEL_VERBOSE
    else:
        configuration['loglevel'] = log.LOGLEVEL_NORMAL
 
    if argdict.has_key('--no-flush-cache'):
	configuration['flush_cache'] = 'no'
    if argdict.has_key('--repository-URI'):
	configuration['sites'] = [('0', argdict['--repository-URI'][0],
				       '.*')]
    if argdict.has_key('--gnupg-dir'):
	configuration['gnupgdir'] = argdict['--gnupg-dir'][0]

    if argdict.has_key('--import-key'):
	filename = argdict['--import-key'][0]
	if os.path.isfile(filename):
	    try:
		import gpg
		(retval, output) = gpg.import_key(filename,
						  configuration['gnupgdir'])
		sys.stdout.write(
		    "Import of key(s) succeded.\n")
		sys.exit(0)
	    except gpg.key_import_failure, errmsg:
		sys.stderr.write(errmsg+'\n')
		sys.exit(1)
	    else:
		sys.stderr.write("No such file '%s'\n" % filename)
		sys.exit(1)

    if argdict.has_key('--list-keys'):
	try:
	    import gpg
	    (retval, output) = gpg.list_keys(
		configuration['gnupgdir'])
	    sys.stdout.write(output)
	    sys.exit(0)
	except gpg.list_keys_failure, errmsg:
	    sys.stderr.write(errmsg+'\n')
	    sys.exit(1)


    # print "Tingeling!\n", configuration
    poll_only = false
    download_only = false

    if argdict.has_key('--poll-only'):
	configuration['poll_only'] = 'yes'
    if argdict.has_key('--download-only'):
	configuration['download_only'] = 'yes'
	configuration['flush_cache'] = 'no'
    if argdict.has_key('--save-to'):
        todir = argdict['--save-to'][0]
        if not os.path.isdir( todir ):
            sys.stderr.write("No such directory '%s'\n" % todir)
            sys.exit(1)
        else:
            configuration['save_to'] = todir

    packages = []

    if argdict.has_key('--package'):
	packages.extend(argdict['--package'])
    if argdict.has_key('--package-file'):
	try:
	    package_file = open(argdict['--package-file'][0])
	    package_file_list = package_file.readlines()
	    #package_file_list.remove('')
	    packages.extend(package_file_list)
	    package_file.close()
	except: 
	    print "swup: Could not open file:",
	    print argdict['--package-file'][0]
	    sys.exit(1)

    if argdict.has_key('--package-URI'):
	packages.extend(argdict['--package-URI'])

    if argdict.has_key('--package-URI-file'):
	try:
	    package_uri_file = \
			     open(argdict['--package-URI-file'][0])
	    package_uri_file_list = package_uri_file.readlines()
	    #package_uri_file_list.remove("")
	    packages.extend(package_uri_file_list)
	    package_uri_file.close()
	except: 
	    print "swup: Could not open URI-file:",
	    print argdict['--package-URI-file'][0]
	    sys.exit(1)

    # prettify list
    for i in range(len(packages)):
	packages[i] = string.strip(packages[i])

    # create the installation object
    installobject = None

    if argdict.has_key('--upgrade'):
	import upgrade
	if packages == []:
	    installobject = upgrade.upgrade_normal(configuration)
	else:
	    installobject = \
			  upgrade.upgrade_package(packages,
						  configuration)

    elif argdict.has_key('--install'):
	if packages == []:
	    print "swup: install: You need to specify", 
	    print "at least one package to install."
	    sys.exit(2)

        import upgrade
        installobject = upgrade.install_package(packages,
                                                configuration)
    elif argdict.has_key('--list-latest'):
	import upgrade
	installobject = upgrade.list_latest(configuration)

    elif argdict.has_key('--list-new'):
        import upgrade
	installobject = upgrade.list_new(configuration)
        
    else:
	print "This should not happen."
	sys.exit(1)
    # Mr. Installobject, take us up
    installobject.run()
    sys.exit(0)

if __name__ == '__main__':
    try:
	main(sys.argv[1:])
    except KeyboardInterrupt:
	sys.stderr.write("User interrupt - exiting.\n")
	sys.stderr.flush()
	sys.exit(1)




